@@ -1,11 +1,11 @@
 ;;; ob-clojure.el --- org-babel functions for clojure evaluation
 
-;; Copyright (C) 2009, 2010  Free Software Foundation, Inc.
+;; Copyright (C) 2009-2011  Free Software Foundation, Inc.
 
-;; Author: Joel Boehland, Eric Schulte
+;; Author: Joel Boehland
+;;	Eric Schulte
 ;; Keywords: literate programming, reproducible research
 ;; Homepage: http://orgmode.org
-;; Version: 7.6
 
 ;; This file is part of GNU Emacs.
 
@@ -62,32 +62,36 @@
 			     vars "\n      ")
 			    "]\n" body ")")
 		  body))))
-    (if (or (member "code" result-params)
-	    (member "pp" result-params))
-	(format
-	 (concat
-	  "(let [org-mode-print-catcher (java.io.StringWriter.)] "
-	  "(clojure.pprint/with-pprint-dispatch clojure.pprint/%s-dispatch "
-	  "(clojure.pprint/pprint (do %s) org-mode-print-catcher) "
-	  "(str org-mode-print-catcher)))")
-	 (if (member "code" result-params) "code" "simple") body)
-      body)))
+    (cond ((or (member "code" result-params) (member "pp" result-params))
+	   (format (concat "(let [org-mode-print-catcher (java.io.StringWriter.)] "
+			   "(clojure.pprint/with-pprint-dispatch clojure.pprint/%s-dispatch "
+			   "(clojure.pprint/pprint (do %s) org-mode-print-catcher) "
+			   "(str org-mode-print-catcher)))")
+		   (if (member "code" result-params) "code" "simple") body))
+	  ;; if (:results output), collect printed output
+	  ((member "output" result-params)
+	   (format "(clojure.core/with-out-str %s)" body))
+	  (t body))))
 
 (defun org-babel-execute:clojure (body params)
   "Execute a block of Clojure code with Babel."
   (require 'slime) (require 'swank-clojure)
   (with-temp-buffer
     (insert (org-babel-expand-body:clojure body params))
-    ((lambda (result) (condition-case nil
-		     (read (org-babel-script-escape result 'force))
-		   (error result)))
+    ((lambda (result)
+       (let ((result-params (cdr (assoc :result-params params))))
+	 (if (or (member "scalar" result-params)
+		 (member "verbatim" result-params))
+	     result
+	   (condition-case nil (org-babel-script-escape result)
+	     (error result)))))
      (slime-eval
       `(swank:interactive-eval-region
-	,(buffer-substring-no-properties (point-min) (point-max)))
+     	,(buffer-substring-no-properties (point-min) (point-max)))
       (cdr (assoc :package params))))))
 
 (provide 'ob-clojure)
 
-;; arch-tag: a43b33f2-653e-46b1-ac56-2805cf05b7d1
+
 
 ;;; ob-clojure.el ends here
