@@ -1,11 +1,10 @@
 ;;; ob-gnuplot.el --- org-babel functions for gnuplot evaluation
 
-;; Copyright (C) 2009, 2010  Free Software Foundation, Inc.
+;; Copyright (C) 2009-2011  Free Software Foundation, Inc.
 
 ;; Author: Eric Schulte
 ;; Keywords: literate programming, reproducible research
 ;; Homepage: http://orgmode.org
-;; Version: 7.6
 
 ;; This file is part of GNU Emacs.
 
@@ -149,7 +148,10 @@ This function is called by `org-babel-execute-src-block'."
                   (shell-command-to-string
 		   (format
 		    "gnuplot \"%s\""
-		    (org-babel-process-file-name script-file))))
+		    (org-babel-process-file-name
+		     script-file
+		     (if (member system-type '(cygwin windows-nt ms-dos))
+			 t nil)))))
             (message output))
         (with-temp-buffer
           (insert (concat body "\n"))
@@ -230,6 +232,6 @@ Pass PARAMS through to `orgtbl-to-generic' when exporting TABLE."
 
 (provide 'ob-gnuplot)
 
-;; arch-tag: 50490ace-a9e1-4b29-a6e5-0db9f16c610b
+
 
 ;;; ob-gnuplot.el ends here
