@@ -1,11 +1,10 @@
 ;;; ob-sqlite.el --- org-babel functions for sqlite database interaction
 
-;; Copyright (C) 2010  Free Software Foundation
+;; Copyright (C) 2010-2011  Free Software Foundation
 
 ;; Author: Eric Schulte
 ;; Keywords: literate programming, reproducible research
 ;; Homepage: http://orgmode.org
-;; Version: 7.6
 
 ;; This file is part of GNU Emacs.
 
@@ -89,6 +88,7 @@ This function is called by `org-babel-execute-src-block'."
 	;; body of the code block
 	(org-babel-expand-body:sqlite body params)))
       (if (or (member "scalar" result-params)
+	      (member "verbatim" result-params)
 	      (member "html" result-params)
 	      (member "code" result-params)
 	      (equal (point-min) (point-max)))
@@ -144,6 +144,6 @@ Prepare SESSION according to the header arguments specified in PARAMS."
 
 (provide 'ob-sqlite)
 
-;; arch-tag: 5c03d7f2-0f72-48b8-bbd1-35aafea248ac
+
 
 ;;; ob-sqlite.el ends here
