@@ -1,12 +1,11 @@
 ;;; org-bibtex.el --- Org links to BibTeX entries
 ;;
-;; Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+;; Copyright (C) 2007-2011 Free Software Foundation, Inc.
 ;;
 ;; Author: Bastien Guerry <bzg at altern dot org>
 ;;         Carsten Dominik <carsten dot dominik at gmail dot com>
 ;;         Eric Schulte <schulte dot eric at gmail dot com>
 ;; Keywords: org, wp, remember
-;; Version: 7.6
 ;;
 ;; This file is part of GNU Emacs.
 ;;
@@ -121,6 +120,7 @@
 (declare-function bibtex-parse-entry "bibtex" (&optional content))
 (declare-function bibtex-url "bibtex" (&optional pos no-browse))
 (declare-function longlines-mode "longlines" (&optional arg))
+(declare-function org-babel-trim "ob" (string &optional regexp))
 
 
 ;;; Bibtex data
@@ -214,7 +214,7 @@
   "List to hold parsed bibtex entries.")
 
 (defcustom org-bibtex-autogen-keys nil
-  "Set to a truthy value to use `bibtex-generate-autokey' to generate keys."
+  "Set to a truth value to use `bibtex-generate-autokey' to generate keys."
   :group 'org-bibtex
   :type  'boolean)
 
@@ -274,11 +274,20 @@ This variable is relevant only if `org-bibtex-export-tags-as-keywords` is t."
   :group 'org-bibtex
   :type '(repeat :tag "Tag" (string)))
 
+(defcustom org-bibtex-type-property-name "btype"
+  "Property in which to store bibtex entry type (e.g., article)."
+  :group 'org-bibtex
+  :type 'string)
+
 
 ;;; Utility functions
 (defun org-bibtex-get (property)
-  (or (org-entry-get (point) (upcase property))
-      (org-entry-get (point) (concat org-bibtex-prefix (upcase property)))))
+  ((lambda (it) (when it (org-babel-trim it)))
+   (let ((org-special-properties
+	  (delete "FILE" (copy-sequence org-special-properties))))
+     (or
+      (org-entry-get (point) (upcase property))
+      (org-entry-get (point) (concat org-bibtex-prefix (upcase property)))))))
 
 (defun org-bibtex-put (property value)
   (let ((prop (upcase (if (keywordp property)
@@ -301,7 +310,7 @@ This variable is relevant only if `org-bibtex-export-tags-as-keywords` is t."
                                    lsts))))
     (let ((notes (buffer-string))
           (id (org-bibtex-get org-bibtex-key-property))
-          (type (org-bibtex-get "type"))
+          (type (org-bibtex-get org-bibtex-type-property-name))
 	  (tags (when org-bibtex-tags-are-keywords
 		  (delq nil
 			(mapcar
@@ -315,16 +324,20 @@ This variable is relevant only if `org-bibtex-export-tags-as-keywords` is t."
         (let ((entry (format
                       "@%s{%s,\n%s\n}\n" type id
                       (mapconcat
-                       (lambda (pair) (format "  %s={%s}" (car pair) (cdr pair)))
+                       (lambda (pair)
+			 (format "  %s={%s}" (car pair) (cdr pair)))
                        (remove nil
 			 (if (and org-bibtex-export-arbitrary-fields
 				  org-bibtex-prefix)
 			     (mapcar
 			      (lambda (kv)
 				(let ((key (car kv)) (val (cdr kv)))
-				  (when (and (string-match org-bibtex-prefix key)
-					     (not (string=
-						   (downcase (concat org-bibtex-prefix "TYPE")) (downcase key))))
+				  (when (and
+					 (string-match org-bibtex-prefix key)
+					 (not (string=
+					       (downcase (concat org-bibtex-prefix
+								 org-bibtex-type-property-name))
+					       (downcase key))))
 				    (cons (downcase (replace-regexp-in-string
 						     org-bibtex-prefix "" key))
 					  val))))
@@ -512,9 +525,20 @@ Headlines are exported using `org-bibtex-export-headline'."
 	  "Bibtex file: " nil nil nil
 	  (file-name-nondirectory
 	   (concat (file-name-sans-extension (buffer-file-name)) ".bib")))))
-  (let ((bibtex-entries (remove nil (org-map-entries #'org-bibtex-headline))))
-    (with-temp-file filename
-      (insert (mapconcat #'identity bibtex-entries "\n")))))
+  ((lambda (error-point)
+     (when error-point
+       (goto-char error-point)
+       (message "Bibtex error at %S" (nth 4 (org-heading-components)))))
+   (catch 'bib
+     (let ((bibtex-entries (remove nil (org-map-entries
+					(lambda ()
+					  (condition-case foo
+					      (org-bibtex-headline)
+					    (error (throw 'bib (point)))))))))
+       (with-temp-file filename
+	 (insert (mapconcat #'identity bibtex-entries "\n")))
+       (message "Successfully exported %d bibtex entries to %s"
+		(length bibtex-entries) filename) nil))))
 
 (defun org-bibtex-check (&optional optional)
   "Check the current headline for required fields.
@@ -523,7 +547,7 @@ With prefix argument OPTIONAL also prompt for optional fields."
   (save-restriction
     (org-narrow-to-subtree)
     (let ((type ((lambda (name) (when name (intern (concat ":" name))))
-                 (org-bibtex-get "TYPE"))))
+                 (org-bibtex-get org-bibtex-type-property-name))))
       (when type (org-bibtex-fleshout type optional)))))
 
 (defun org-bibtex-check-all (&optional optional)
@@ -540,7 +564,8 @@ If nonew is t, add data to the headline of the entry at point."
 		"Type: " (mapcar (lambda (type)
 				   (substring (symbol-name (car type)) 1))
 				 org-bibtex-types)
-		nil nil (when nonew (org-bibtex-get "TYPE"))))
+		nil nil (when nonew
+			  (org-bibtex-get org-bibtex-type-property-name))))
 	 (type (if (keywordp type) type (intern (concat ":" type))))
 	 (org-bibtex-treat-headline-as-title (if nonew nil t)))
     (unless (assoc type org-bibtex-types)
@@ -551,9 +576,10 @@ If nonew is t, add data to the headline of the entry at point."
       (let ((title (org-bibtex-ask :title)))
 	(insert title)
 	(org-bibtex-put "TITLE" title)))
-    (org-bibtex-put "TYPE" (substring (symbol-name type) 1))
+    (org-bibtex-put org-bibtex-type-property-name
+		    (substring (symbol-name type) 1))
     (org-bibtex-fleshout type arg)
-    (mapc (lambda (tag) (org-toggle-tag tag t)) org-bibtex-tags)))
+    (mapc (lambda (tag) (org-toggle-tag tag 'on)) org-bibtex-tags)))
 
 (defun org-bibtex-create-in-current-entry (&optional arg)
   "Add bibliographical data to the current entry.
@@ -592,11 +618,11 @@ This uses `bibtex-parse-entry'."
   (let ((entry (pop *org-bibtex-entries*))
 	(org-special-properties nil)) ; avoids errors with `org-entry-put'
     (flet ((val (field) (cdr (assoc field entry)))
-	   (togtag (tag) (org-toggle-tag tag t)))
+	   (togtag (tag) (org-toggle-tag tag 'on)))
       (org-insert-heading)
       (insert (val :title))
       (org-bibtex-put "TITLE" (val :title))
-      (org-bibtex-put "TYPE"  (downcase (val :type)))
+      (org-bibtex-put org-bibtex-type-property-name (downcase (val :type)))
       (dolist (pair entry)
         (case (car pair)
           (:title    nil)
@@ -635,11 +661,10 @@ This function relies `org-search-view' to locate results."
   (let ((org-agenda-overriding-header "Bib search results:")
         (org-agenda-search-view-always-boolean t))
     (org-search-view nil
-		     (format "%s +{:%sTYPE:}"
-			     string org-bibtex-prefix))))
+		     (format "%s +{:%s%s:}"
+			     string org-bibtex-prefix
+			     org-bibtex-type-property-name))))
 
 (provide 'org-bibtex)
 
-;; arch-tag: 83987d5a-01b8-41c7-85bc-77700f1285f5
-
 ;;; org-bibtex.el ends here
